/*****************************************
    Bluetooth remote controlled Spider Robot

    V1.3
    By Matija Hardi 2021
*****************************************/
#include <Servo.h>                        //adds the servo library

#define leftMotor 5                       //defining the pins
#define rightMotor 6
#define shooterMotor 7
#define laserPin 8
#define turretMotor 9

char Incoming_value;                      //Variable for storing the data received over bluetooth
bool laser = true;                        //Variable to store the laser state, true for on, false for off

long timeOfShooting;                      //variable that stores the time when the shooting motor was activated
int shooting = 600;                       //for how long you want the motor to shoot, 500ms is about two revolutions

long sinceMotion;                         //variable that stores the time when walking motors were engaged
int moving = 200;                         //for how long the motors are turned on after receiveing the signal, the app resends commands every 100ms so this should be a little longer

int forwardSpeed = 180;                   //the speed at which the continuous rotation motors spin, 90 is 0% and 0/180 is 100%
int backwardSpeed = 0;
int upLimit = 70;                         //the biggest angle that the turret can swing to, normaly 45 but adjust to your setup
int downLimit = 140;                      //the smallest angle that the turret can swing to, normaly 135 but adjust to your setup
int turretAngle = 105;                    //the middle angle, normaly 90 but adjust to halfway between your up and down limit

Servo leftServo;                          //creates servo objects for each servo
Servo rightServo;
Servo turretServo;

void setup()
{
  Serial.begin(9600);                     //Sets the data rate in bits per second (baud) for serial data transmission
  pinMode(shooterMotor, OUTPUT);          //Sets the pins as output
  pinMode(laserPin, OUTPUT);
  leftServo.attach(leftMotor);            //attaches servo objects to their pins
  rightServo.attach(rightMotor);
  turretServo.attach(turretMotor);

  digitalWrite(laserPin, laser);          //turns of the laser on startup
  turretServo.write(turretAngle);         //sets the turret to center angle on startup
}
void loop()
{
  while (Serial.available() > 0) {            //checks to see if serial is open
    Incoming_value = Serial.read();           //Read the incoming data and store it into variable Incoming_value
    //Serial.println(Incoming_value);         //Print Value of Incoming_value in Serial monitor, uncomment for debugging

    switch (Incoming_value) {                 //switch function with incoming bluetooth value as input
      case 'X' :                              //checks to see if received character is the same as set, adjust to your smartphone app
        timeOfShooting = millis();            //stores the current board time in the variable
        digitalWrite(shooterMotor, HIGH);     //turns on the motor
        Incoming_value = ' ';                 //clears the variable so that it does not repeat the commands
        break;                                //exits the case
      case 'Z' :
        laser = !laser;                       //toggles the laser variable, sets true to false and vice versa
        digitalWrite(laserPin, laser);        //sets the pin to the new state
        Incoming_value = ' ';
        break;
      case 'F' :
        sinceMotion = millis();
        leftServo.write(forwardSpeed);        //sets the servo motors to the set speed
        rightServo.write(backwardSpeed);      //for straght line motion motors are set to opposite directions
        Incoming_value = ' ';
        break;
      case 'B' :
        sinceMotion = millis();
        leftServo.write(backwardSpeed);
        rightServo.write(forwardSpeed);
        Incoming_value = ' ';
        break;
      case 'L' :
        sinceMotion = millis();
        leftServo.write(backwardSpeed);       //for turning motion the motors are set to the same direction
        rightServo.write(backwardSpeed);
        Incoming_value = ' ';
        break;
      case 'R' :
        sinceMotion = millis();
        leftServo.write(forwardSpeed);
        rightServo.write(forwardSpeed);
        Incoming_value = ' ';
        break;
      case 'U' :
        if (turretAngle > upLimit) {          //checks if the turret is at maximum angle
          turretAngle -= 2;                   //if it is not the angle is increased
        }
        turretServo.write(turretAngle);       //the new angle is set to the servo
        //Serial.println(turretAngle);
        Incoming_value = ' ';
        break;
      case 'D' :
        if (turretAngle < downLimit) {        //checks if the turret is at minimum angle
          turretAngle += 2;                   //if it is not the angle is decreased
        }
        turretServo.write(turretAngle);
        //Serial.println(turretAngle);
        Incoming_value = ' ';
        break;
    }
  }

  if (millis() >= (timeOfShooting + shooting)) {    //checks to see if enough time has elapsed since the motor was turned on
    digitalWrite(shooterMotor, LOW);                //turns the motor of when it has spun enough
  }
  if (millis() >= (sinceMotion + moving)) {         //checks to see if enough time has elapsed since motion started
    leftServo.write(90);                            //sets the servo speed to zero
    rightServo.write(90);
  }

}
